@@ -99,12 +99,14 @@ class NaPoleonBinanceFutureBot(object):
         positions ={}
         if pair == 'BTCUSDT':
             positions['BTC']=pair_signal
+            self.make_sure_positions_changed(positions)
         else :
             print('pair not handled for the moment '+pair)
-        self.change_position(positions)
+
 
 
     def change_position(self, positions):
+        print('incoming positions to hodl ',str(positions))
         usdt_balance, balance, ticker_data, minBTC, minETH, minLTC, minXRP, minEOS, minTRX, minBCH = self.get_balance()
         ETH = float(ticker_data['ETH/USDT']['info']['lastPrice'])
         BTC = float(ticker_data['BTC/USDT']['info']['lastPrice'])
