@@ -26,14 +26,17 @@ import torch.nn as nn
 
 
 class AbstractRunner(ABC):
-    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, save_model=False, supervision_npy_file_suffix='_supervision.npy', macro_supervision_npy_file_suffix='_macro_supervision.npy', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy', shortcut_features_names_saving_suffix = '_shortcut_features_names.npy', shortcut_features_saving_suffix='_shortcut_features.npy',returns_pkl_file_name='returns.pkl',  local_root_directory='../data/', user = 'napoleon', n_start = 252, maximum_precision = False, shortcut_features = False, transaction_costs = None):
+    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, save_model=False, supervision_npy_file_suffix='_supervision.npy', macro_supervision_npy_file_suffix='_macro_supervision.npy', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy', shortcut_features_names_saving_suffix = '_shortcut_features_names.npy', shortcut_features_saving_suffix='_shortcut_features.npy',returns_pkl_file_suffix='_returns.pkl',  local_root_directory='../data/', user = 'napoleon', n_start = 252, maximum_precision = False, shortcut_features = False, transaction_costs = None):
         super().__init__()
         self.local_root_directory=local_root_directory
         self.supervision_npy_file_suffix=supervision_npy_file_suffix
         self.macro_supervision_npy_file_suffix=macro_supervision_npy_file_suffix
         self.features_saving_suffix=features_saving_suffix
         self.features_names_saving_suffix=features_names_saving_suffix
-        self.returns_pkl_file_name=returns_pkl_file_name
+
+        self.returns_pkl_file_suffix=returns_pkl_file_suffix
+        self.returns_pkl_file_name = running_date.strftime('%d_%b_%Y')+returns_pkl_file_suffix
+
         self.shortcut_features_saving_suffix = shortcut_features_saving_suffix
         self.shortcut_features_names_saving_suffix = shortcut_features_names_saving_suffix
         self.shortcut_features = shortcut_features
