@@ -1154,7 +1154,7 @@ class RealTimeSignalEnsemblingParalleLauncher():
         if  self.day_signal_np.shape[0]> self.daily_ia_lookback_window + 1:
             discrete_signals = self.daily_apply_standard_mean_two_states_discretization_last_signal()
         else :
-            return np.nanmean(last_day_signals)
+            return last_day_ts,np.nanmean(last_day_signals)
         return last_day_ts, discrete_signals
 
 
@@ -1170,7 +1170,7 @@ class RealTimeSignalEnsemblingParalleLauncher():
         if  self.minute_signal_np.shape[0]> self.minutely_ia_lookback_window + 1:
             discrete_signals = self.minutely_apply_standard_mean_two_states_discretization_last_signal()
         else :
-            return np.nanmean(last_minute_signals)
+            return last_minute_ts, np.nanmean(last_minute_signals)
         return last_minute_ts, discrete_signals
 
 
