@@ -18,8 +18,9 @@ from napoleontoolbox.utility import metrics
 continuum_signals = ['alpha_2', 'alpha_3', 'alpha5',  'alpha_6', 'alpha_6_rank', 'alpha_8', 'alpha_12', 'alpha_13', 'alpha_14',  'alpha_15',  'alpha_16', 'slope_induced' ]
 
 def compute_correlation(col_one, col_two):
-    a = ma.masked_invalid(col_one)
-    b = ma.masked_invalid(col_two)
+
+    a = ma.masked_invalid(col_one.astype(float))
+    b = ma.masked_invalid(col_two.astype(float))
     msk = (~a.mask & ~b.mask)
     correlation_matrix = ma.corrcoef(col_one[msk], col_two[msk])
     correlation_coefficient = correlation_matrix[0][1]
@@ -915,7 +916,7 @@ def slope_lo_to_ls(data = None, contravariant = -1., up_trend_threshold=0.8, up_
     slope_rec_close = np.nan
     if slope_column == 'close':
         y = data[slope_column][-pente_window:].values
-        slope_close = stats.linregress(np.arange(pente_window), y).slope
+        slope_close = stats.linregress(np.arange(pente_window), y.astype(float)).slope
 
         y_rec = data['rec_close'][-pente_window:].values
         slope_rec_close = stats.linregress(np.arange(pente_window), y_rec).slope
@@ -926,7 +927,6 @@ def slope_lo_to_ls(data = None, contravariant = -1., up_trend_threshold=0.8, up_
     elif slope_column == 'high':
         raise Exception('not implemented yet')
 
-
     slope_close_condition = (slope_close>0 )
     slope_rec_close_condition = (slope_rec_close > 0)
     slope_opp_close_condition = (slope_opp_close > 0)
