@@ -11,7 +11,7 @@ from napoleontoolbox.connector import napoleon_connector
 import json
 
 from napoleontoolbox.parallel_run import signal_result_analyzer
-
+from sklearn.metrics import accuracy_score
 from functools import partial
 
 
@@ -415,4 +415,288 @@ def daily_signal_hourlyzation(close, daily_signal, transaction_cost=True, shift
     hourly_signal = hourly_signal.loc[hourly_signal.index <= running_date]
     perf_df = reconstitute_prediction_perf(y_pred=hourly_signal, y_true=close, initial_price = 1. , average_execution_cost = 7.5e-4 , transaction_cost = transaction_cost, print_turnover = False)
     #new_signals = perf_df[['reconstituted_close', 'reconstituted_perf']]
-    return perf_df
\ No newline at end of file
+    return perf_df
+
+
+def apply_discretization_two_states_lo(y_pred, splitting_threshold):
+    y_pred_discrete = np.zeros(y_pred.shape)
+    if np.isnan(splitting_threshold) :
+        splitting_threshold = 0.5
+    y_pred_discrete[y_pred > splitting_threshold] = 1.
+    y_pred_discrete[y_pred < splitting_threshold] = 0.
+    return y_pred_discrete
+
+def discretize_two_states_lo(y_pred, splitting_quantile_threshold=0.5):
+    y_pred_discrete = np.zeros(y_pred.shape)
+    threshold = np.nan
+    try:
+        threshold = np.quantile(y_pred, splitting_quantile_threshold)
+    except Exception as e:
+        print(e)
+        print("the quantile could not get computed")
+    if np.isnan(threshold) :
+        threshold = 0.5
+    y_pred_discrete[y_pred > threshold] = 1.
+    y_pred_discrete[y_pred < threshold] = 0.
+    return y_pred_discrete
+
+def apply_discretization_three_states(y_pred, upper_threshold, lower_threshold):
+    y_pred_discrete = np.zeros(y_pred.shape)
+
+    if np.isnan(upper_threshold):
+        upper_threshold = 0.8
+    if np.isnan(lower_threshold):
+        lower_threshold = 0.2
+
+    y_pred_discrete[y_pred > upper_threshold] = 1.
+    y_pred_discrete[y_pred < lower_threshold] = -1.
+    return y_pred_discrete
+
+
+def discretize_three_states(y_pred, upper_quantile_threshold=0.8, lower_quantile_threshold = 0.2):
+    y_pred_discrete = np.zeros(y_pred.shape)
+
+    lower_threshold = np.nan
+    try:
+        lower_threshold = np.quantile(y_pred, lower_quantile_threshold)
+    except Exception as e:
+        print(e)
+        print("the quantile could not get computed")
+
+    upper_threshold = np.nan
+    try:
+        upper_threshold = np.quantile(y_pred, upper_quantile_threshold)
+    except Exception as e:
+        print(e)
+        print("the quantile could not get computed")
+
+    if np.isnan(upper_threshold):
+        upper_threshold = 0.8
+    if np.isnan(lower_threshold):
+        lower_threshold = 0.2
+
+    y_pred_discrete[y_pred > upper_threshold] = 1.
+    y_pred_discrete[y_pred < lower_threshold] = -1.
+    return y_pred_discrete
+
+
+def apply_discretization_three_states_lo(y_pred, upper_threshold, lower_threshold):
+    y_pred_discrete = np.ones(y_pred.shape)/2
+
+    if np.isnan(upper_threshold):
+        upper_threshold = 0.8
+    if np.isnan(lower_threshold):
+        lower_threshold = 0.2
+
+    y_pred_discrete[y_pred > upper_threshold] = 1.
+    y_pred_discrete[y_pred < lower_threshold] = 0.
+    return y_pred_discrete
+
+
+def discretize_three_states_lo(y_pred, upper_quantile_threshold=0.8, lower_quantile_threshold = 0.2):
+    y_pred_discrete = np.ones(y_pred.shape)/2
+
+    lower_threshold = np.nan
+    try:
+        lower_threshold = np.quantile(y_pred, lower_quantile_threshold)
+    except Exception as e:
+        print(e)
+        print("the quantile could not get computed")
+
+    upper_threshold = np.nan
+    try:
+        upper_threshold = np.quantile(y_pred, upper_quantile_threshold)
+    except Exception as e:
+        print(e)
+        print("the quantile could not get computed")
+
+    if np.isnan(upper_threshold):
+        upper_threshold = 0.8
+    if np.isnan(lower_threshold):
+        lower_threshold = 0.2
+
+    y_pred_discrete[y_pred > upper_threshold] = 1.
+    y_pred_discrete[y_pred < lower_threshold] = 0.
+    return y_pred_discrete
+
+def apply_discretization_two_states(y_pred, splitting_threshold):
+    y_pred_discrete = np.zeros(y_pred.shape)
+    if np.isnan(splitting_threshold) :
+        splitting_threshold = 0.5
+    y_pred_discrete[y_pred > splitting_threshold] = 1.
+    y_pred_discrete[y_pred < splitting_threshold] = -1.
+    return y_pred_discrete
+
+def discretize_two_states(y_pred, splitting_quantile_threshold=0.5):
+    y_pred_discrete = np.zeros(y_pred.shape)
+    threshold = np.nan
+    try:
+        threshold = np.quantile(y_pred, splitting_quantile_threshold)
+    except Exception as e:
+        print(e)
+        print("the quantile could not get computed")
+    if np.isnan(threshold) :
+        threshold = 0.5
+    y_pred_discrete[y_pred > threshold] = 1.
+    y_pred_discrete[y_pred < threshold] = -1.
+    return y_pred_discrete
+
+def find_best_two_states_discretization(y_pred, y, seed = 42, number_per_year = 252):
+    N_ = 1
+    w0 = np.array([0.5])
+    # Set constraints
+    #const_sum = LinearConstraint(np.ones([1, N_]), [1], [1])
+    up_bound_ = 1.
+    low_bound_ = 0.
+    const_ind = Bounds(low_bound_ * np.ones([N_]), up_bound_ * np.ones([N_]))
+    def f_to_optimize(w):
+        y_pred_discrete = discretize_two_states(y_pred, splitting_quantile_threshold=w[0])
+        perf_df = signal_utility.reconstitute_prediction_perf(y_pred=y_pred_discrete, y_true=y,
+                                                              transaction_cost=False,
+                                                              print_turnover=False)
+        sharpe_strat = metrics.sharpe(perf_df['perf_return'].dropna(), period=number_per_year, from_ret=True)
+        return sharpe_strat
+
+    np.random.seed(seed)
+    # Optimize f
+    w__ = minimize(
+        f_to_optimize,
+        w0,
+        method='SLSQP',
+        #constraints=[const_sum],
+        bounds=const_ind
+    ).x
+
+    splitting_quantile_threshold = w__[0]
+    y_pred_discrete = discretize_two_states(y_pred, splitting_quantile_threshold=splitting_quantile_threshold)
+    perf_df = signal_utility.reconstitute_prediction_perf(y_pred=y_pred_discrete, y_true=y,
+                                                          transaction_cost=False,
+                                                          print_turnover=False)
+    sharpe_strat = metrics.sharpe(perf_df['perf_return'].dropna(), period=number_per_year, from_ret=True)
+    sharpe_under = metrics.sharpe(perf_df['close_return'].dropna(), period=number_per_year, from_ret=True)
+    splitting_threshold = np.quantile(y_pred, splitting_quantile_threshold)
+    return splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under
+
+def find_best_two_states_discretization_lo(y_pred, y, seed = 42, number_per_year = 252):
+    N_ = 1
+    w0 = np.array([0.5])
+    # Set constraints
+    #const_sum = LinearConstraint(np.ones([1, N_]), [1], [1])
+    up_bound_ = 1.
+    low_bound_ = 0.
+    const_ind = Bounds(low_bound_ * np.ones([N_]), up_bound_ * np.ones([N_]))
+    def f_to_optimize(w):
+        y_pred_discrete = discretize_two_states_lo(y_pred, splitting_quantile_threshold=w[0])
+        perf_df = signal_utility.reconstitute_prediction_perf(y_pred=y_pred_discrete, y_true=y,
+                                                              transaction_cost=False,
+                                                              print_turnover=False)
+        sharpe_strat = metrics.sharpe(perf_df['perf_return'].dropna(), period=number_per_year, from_ret=True)
+        return sharpe_strat
+
+    np.random.seed(seed)
+    # Optimize f
+    w__ = minimize(
+        f_to_optimize,
+        w0,
+        method='SLSQP',
+        #constraints=[const_sum],
+        bounds=const_ind
+    ).x
+
+    splitting_quantile_threshold = w__[0]
+    y_pred_discrete = discretize_two_states_lo(y_pred, splitting_quantile_threshold=splitting_quantile_threshold)
+    perf_df = signal_utility.reconstitute_prediction_perf(y_pred=y_pred_discrete, y_true=y,
+                                                          transaction_cost=False,
+                                                          print_turnover=False)
+    sharpe_strat = metrics.sharpe(perf_df['perf_return'].dropna(), period=number_per_year, from_ret=True)
+    sharpe_under = metrics.sharpe(perf_df['close_return'].dropna(), period=number_per_year, from_ret=True)
+    splitting_threshold = np.quantile(y_pred, splitting_quantile_threshold)
+    return splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under
+
+def find_best_three_states_discretization(y_pred, y, seed = 42, number_per_year=252):
+    N_ = 2
+    w0 = np.array([0.3,0.7])
+    # Set constraints
+    #const_sum = LinearConstraint(np.ones([1, N_]), [1], [1])
+    up_bound_ = 1.
+    low_bound_ = 0.
+    const_ind = Bounds(low_bound_ * np.ones([N_]), up_bound_ * np.ones([N_]))
+
+    def f_to_optimize(w):
+        lower_quantile_threshold = w[0]
+        upper_quantile_threshold = w[1]
+
+        y_pred_discrete = discretize_three_states(y_pred, upper_quantile_threshold=upper_quantile_threshold, lower_quantile_threshold=lower_quantile_threshold)
+        perf_df = signal_utility.reconstitute_prediction_perf(y_pred=y_pred_discrete, y_true=y,
+                                                              transaction_cost=False,
+                                                              print_turnover=False)
+        sharpe_strat = metrics.sharpe(perf_df['perf_return'].dropna(), period=number_per_year, from_ret=True)
+        return sharpe_strat
+
+    np.random.seed(seed)
+    # Optimize f
+    w__ = minimize(
+        f_to_optimize,
+        w0,
+        method='SLSQP',
+        #constraints=[const_sum],
+        bounds=const_ind
+    ).x
+
+    lower_quantile_threshold = w__[0]
+    upper_quantile_threshold = w__[1]
+    y_pred_discrete = discretize_three_states(y_pred, upper_quantile_threshold=upper_quantile_threshold,
+                                 lower_quantile_threshold=lower_quantile_threshold)
+    perf_df = signal_utility.reconstitute_prediction_perf(y_pred=y_pred_discrete, y_true=y,
+                                                          transaction_cost=False,
+                                                          print_turnover=False)
+    sharpe_strat = metrics.sharpe(perf_df['perf_return'].dropna(), period=number_per_year, from_ret=True)
+    sharpe_under = metrics.sharpe(perf_df['close_return'].dropna(), period=number_per_year, from_ret=True)
+
+    lower_threshold = np.quantile(y_pred, lower_quantile_threshold)
+    upper_threshold = np.quantile(y_pred, upper_quantile_threshold)
+    return lower_quantile_threshold, upper_quantile_threshold, lower_threshold, upper_threshold, sharpe_strat, sharpe_under
+
+def find_best_three_states_discretization_lo(y_pred, y, seed = 42, number_per_year=252):
+    N_ = 2
+    w0 = np.array([0.3,0.7])
+    # Set constraints
+    #const_sum = LinearConstraint(np.ones([1, N_]), [1], [1])
+    up_bound_ = 1.
+    low_bound_ = 0.
+    const_ind = Bounds(low_bound_ * np.ones([N_]), up_bound_ * np.ones([N_]))
+
+    def f_to_optimize(w):
+        lower_quantile_threshold = w[0]
+        upper_quantile_threshold = w[1]
+
+        y_pred_discrete = discretize_three_states_lo(y_pred, upper_quantile_threshold=upper_quantile_threshold, lower_quantile_threshold=lower_quantile_threshold)
+        perf_df = signal_utility.reconstitute_prediction_perf(y_pred=y_pred_discrete, y_true=y,
+                                                              transaction_cost=False,
+                                                              print_turnover=False)
+        sharpe_strat = metrics.sharpe(perf_df['perf_return'].dropna(), period=number_per_year, from_ret=True)
+        return sharpe_strat
+
+    np.random.seed(seed)
+    # Optimize f
+    w__ = minimize(
+        f_to_optimize,
+        w0,
+        method='SLSQP',
+        #constraints=[const_sum],
+        bounds=const_ind
+    ).x
+
+    lower_quantile_threshold = w__[0]
+    upper_quantile_threshold = w__[1]
+    y_pred_discrete = discretize_three_states_lo(y_pred, upper_quantile_threshold=upper_quantile_threshold,
+                                 lower_quantile_threshold=lower_quantile_threshold)
+    perf_df = signal_utility.reconstitute_prediction_perf(y_pred=y_pred_discrete, y_true=y,
+                                                          transaction_cost=False,
+                                                          print_turnover=False)
+    sharpe_strat = metrics.sharpe(perf_df['perf_return'].dropna(), period=number_per_year, from_ret=True)
+    sharpe_under = metrics.sharpe(perf_df['close_return'].dropna(), period=number_per_year, from_ret=True)
+    lower_threshold = np.quantile(y_pred, lower_quantile_threshold)
+    upper_threshold = np.quantile(y_pred, upper_quantile_threshold)
+    return lower_quantile_threshold, upper_quantile_threshold, lower_threshold, upper_threshold, sharpe_strat, sharpe_under
+
